----------------------------------------------------------
-- Example(4-32): Waveform generator 
----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
----------------------------------------------------------
ENTITY waveform IS
   GENERIC ( bits: INTEGER := 4;   --bits per word
	      words: INTEGER := 13); --words in the memory
   port (CLOCK: in BIT; RESET: in BOOLEAN;
         WAVES: out STD_LOGIC_VECTOR (bits-1 DOWNTO 0));
END waveform;
----------------------------------------------------------
ARCHITECTURE BEHAVIOR OF waveform IS
   TYPE vector_array IS ARRAY (0 TO words-1) OF 
	                  STD_LOGIC_VECTOR (bits-1 DOWNTO 0);
   CONSTANT ROM: vector_array :=(	
            "1100",  -- time step 0
            "1100",  -- time step 1
            "0100",  -- time step 2
            "0000",  -- time step 3
            "0110",  -- time step 4
            "0101",  -- time step 5
            "0111",  -- time step 6
            "1100",  -- time step 7
            "0100",  -- time step 8
            "0000",  -- time step 9
            "0110",  -- time step 10
            "0101",  -- time step 11
            "0111"); -- time step 12
   subtype ROM_RANGE is INTEGER range 0 to words-1;
   signal STEP: ROM_RANGE;
begin
   TIMESTEP_COUNTER: process  	--Time stepping process
   begin
      wait until CLOCK'event and CLOCK = '1';
      if RESET then 			-- Detect reset
   	   STEP <= ROM_RANGE'low; 	-- Restart
      elsif STEP = ROM_RANGE'high then 	-- Finished?
   	-- STEP <= ROM_RANGE'high; 	-- Hold at last value
  	   STEP <= ROM_RANGE'low; 	-- Continuous wave
      else STEP <= STEP + 1;		-- Continue stepping
      end if;
   end process TIMESTEP_COUNTER;
   WAVES <= ROM(STEP);
end BEHAVIOR;
----------------------------------------------------------
