-----------------------------------------------------------
-- Example(4-33): Smart Waveform generator 
-----------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
-----------------------------------------------------------
ENTITY waveform IS
   GENERIC(bits: INTEGER := 4;     -- of bits per word
		words: INTEGER := 13); -- words in the memory
   port (CLOCK: in BIT;
         RESET: in BOOLEAN;
   WAVES: out STD_LOGIC_VECTOR (bits-1 DOWNTO 0));
END waveform;
-----------------------------------------------------------
ARCHITECTURE BEHAVIOR OF waveform IS
   TYPE vector_array IS ARRAY (0 TO words-1) OF 
	                  STD_LOGIC_VECTOR (bits-1 DOWNTO 0);
   CONSTANT ROM: vector_array :=(	
"1100",  -- time step 0
			"1100",  -- time step 1
			"0100",  -- time step 2
			"0000",  -- time step 3
			"0110",  -- time step 4
			"0101",  -- time step 5
			"0111",  -- time step 6
			"1100",  -- time step 7
			"0100",  -- time step 8
			"0000",  -- time step 9
			"0110",  -- time step 10
			"0101",  -- time step 11
			"0111"); -- time step 12  
-- a 7x13 ROM called D_ROM containing the delays
   subtype D_ROM_WORD is INTEGER range 0 to 100;
   subtype D_ROM_RANGE is INTEGER range 0 to words-1;
   type D_ROM_TABLE is array (0 to words-1) of D_ROM_WORD;
   constant D_ROM: D_ROM_TABLE 
:=(1,80,5,1,1,1,1,20,5,1,1,1,1);
   subtype ROM_RANGE is INTEGER range 0 to words-1;
   signal STEP, NEXT_STEP: ROM_RANGE;
   signal DELAY: D_ROM_WORD;
begin
-- Determine the value of the next time step
   NEXT_STEP <= ROM_RANGE'high when STEP = ROM_RANGE'high 
    else STEP + 1;
-- Keep track of which time step we are in
   TIMESTEP_COUNTER: process
	
begin 
		wait until CLOCK'event and CLOCK = '1';
		if RESET then 		-- Detect reset
			STEP <= 0; 		-- Restart waveform
		elsif DELAY = 1 then
			STEP <= NEXT_STEP; -- Continue stepping
		else
			null; -- Wait for DELAY to count down;
		end if; 	-- do nothing here
end process TIMESTEP_COUNTER;

-- Count the delay between time steps
	DELAY_COUNTER: process
	begin
		wait until CLOCK'event and CLOCK = '1';
		if RESET then 		--Detect reset
		   DELAY <= D_ROM(0);--Restart
		elsif DELAY = 1 then 	--Have we counted down?
		   DELAY <= D_ROM(NEXT_STEP);--Next delay value
		else
		   DELAY <= DELAY - 1;--decrement DELAY counter
		end if;
	end process DELAY_COUNTER;
	WAVES <= ROM(STEP); 	-- Output waveform value
end BEHAVIOR;
-----------------------------------------------------------
